/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scope/store/package.h"

#include <gtest/gtest.h>
#include <gmock/gmock.h>


namespace
{
using namespace Libertine::Store;


static QVariantMap
attribute_map(QString const& name, QString const& summary, QString const& description,
              QString const& icon, QString const& publisher, QString const& website,
              QString const& license, QString const& id, double rating)
{
  QVariantMap attributes;
  attributes["name"] = name;
  attributes["summary"] = summary;
  attributes["rating"] = rating;
  attributes["description"] = description;
  attributes["icon"] = icon;
  attributes["publisher"] = publisher;
  attributes["website"] = website;
  attributes["license"] = license;
  attributes["id"] = id;
  return attributes;
}


static void
package_matches_attributes(Package actual, std::string const& name, std::string const& summary,
                           std::string const& description, std::string const& icon, std::string const& publisher,
                           std::string const& website, std::string const& license, std::string const& id, double rating)
{
  EXPECT_EQ(name, actual.name);
  EXPECT_EQ(summary, actual.summary);
  EXPECT_EQ(rating, actual.rating);
  EXPECT_EQ(description, actual.description);
  EXPECT_EQ(icon, actual.icon);
  EXPECT_EQ(publisher, actual.publisher);
  EXPECT_EQ(website, actual.website);
  EXPECT_EQ(license, actual.license);
  EXPECT_EQ(id, actual.id);
}


TEST(PackageTest, FromMapCreatesSinglePackage)
{
  auto attributeMap = attribute_map("Harry", "The Philosopher's Stone",
                                    "Boy with head injury seeks magic rock",
                                    "harry.png", "JK", "pottermore.com", "Private",
                                    "com.pottermore.harry", 3.4);
  auto package = Package::from_map(attributeMap);
  package_matches_attributes(package, "Harry", "The Philosopher's Stone",
                             "Boy with head injury seeks magic rock",
                             "harry.png", "JK", "pottermore.com", "Private",
                             "com.pottermore.harry", 3.4);
}


TEST(PackageTest, FromMapCreatesListOfPackages)
{
  auto attributePkg1Map = attribute_map("Harry", "The Philosopher's Stone",
                                    "Boy with head injury seeks magic rock",
                                    "harry.png", "JK", "pottermore.com", "Private",
                                    "com.pottermore.harry", 3.4);
  auto attributePkg2Map = attribute_map("Dune", "Classic science fiction",
                                    "The spice must flow", "worms.png",
                                    "Herbert", "dunebook.net", "Other",
                                    "net.dunebook.dune", 4.1);
  auto packages = Package::from_map(QList<QVariantMap>{attributePkg1Map, attributePkg2Map});
  ASSERT_EQ(2, packages.size());
  package_matches_attributes(packages[0], "Harry", "The Philosopher's Stone",
                             "Boy with head injury seeks magic rock",
                             "harry.png", "JK", "pottermore.com", "Private",
                             "com.pottermore.harry", 3.4);
  package_matches_attributes(packages[1], "Dune", "Classic science fiction",
                             "The spice must flow", "worms.png",
                             "Herbert", "dunebook.net", "Other",
                             "net.dunebook.dune", 4.1);
}


TEST(PackageTest, RenderRatingReturnsMinimum0EmptyStars)
{
  Package p;
  p.rating = -5;
  EXPECT_EQ("☆☆☆☆☆", p.render_rating());
}


TEST(PackageTest, RenderRatingReturnsMaximum5FilledStars)
{
  Package p;
  p.rating = 6;
  EXPECT_EQ("★★★★★", p.render_rating());
}


TEST(PackageTest, RenderRatingReturnsRoundedUpStars)
{
  Package p;
  p.rating = 3.49;
  EXPECT_EQ("★★★☆☆", p.render_rating());
}


TEST(PackageTest, RenderRatingReturnsRoundedDownStars)
{
  Package p;
  p.rating = 3.5;
  EXPECT_EQ("★★★★☆", p.render_rating());
}
}
